@@ -67,6 +67,7 @@ struct hash_name {
 };
 
 struct hash_key {
+	const char *subdir;
 	const uint8_t *data;
 	uint32_t len;
 };
@@ -103,10 +104,12 @@ static const char hmaccalc_hmackey[] = "FIPS-FTW-RHT2009";
 static const struct hash_key KEY_FIPSCHECK = {
 	.data = (const uint8_t *)fipscheck_hmackey,
 	.len = sizeof(fipscheck_hmackey) - 1,
+	.subdir = "fipscheck",
 };
 static const struct hash_key KEY_HMACCALC = {
 	.data = (const uint8_t *)hmaccalc_hmackey,
 	.len = sizeof(hmaccalc_hmackey) - 1,
+	.subdir = "hmaccalc",
 };
 
 static void usage(char *name, int fipscheck)
@@ -114,15 +117,17 @@ static void usage(char *name, int fipscheck)
 	const char *base = basename(name);
 	fprintf(stderr, "\n%s - calculation of hash sum (Using Linux Kernel Crypto API)\n", basename(name));
 	fprintf(stderr, "\nUsage:\n");
-	fprintf(stderr, "\t%s [OPTION]... -S\n", base);
+	fprintf(stderr, "\t%s [-n BASENAME] [OPTION]... -S|-L\n", base);
 	if (fipscheck)
-		fprintf(stderr, "\t%s [OPTION]... FILE\n", base);
+		fprintf(stderr, "\t%s [-n BASENAME] [OPTION]... FILE\n", base);
 	else {
-		fprintf(stderr, "\t%s [OPTION]... -c FILE\n", base);
-		fprintf(stderr, "\t%s [OPTION]... FILE...\n", base);
+		fprintf(stderr, "\t%s [-n BASENAME] [OPTION]... -c FILE\n", base);
+		fprintf(stderr, "\t%s [-n BASENAME] [OPTION]... FILE...\n", base);
 	}
 	fprintf(stderr, "\nOptions:\n");
+	fprintf(stderr, "\t-n --name\t\tForce given application name (sha512hmac/...)\n");
 	fprintf(stderr, "\t-S --self-sum\t\tPrint checksum of this binary and exit\n");
+	fprintf(stderr, "\t-L --self-sum-lib\tPrint checksum of the libkcapi library and exit\n");
 	if (!fipscheck)
 		fprintf(stderr, "\t-c --check FILE\t\tVerify hash sums from file\n");
 	fprintf(stderr, "\t-u --unkeyed\t\tForce unkeyed hash\n");
@@ -348,9 +353,9 @@ static char *paste(char *dst, const char *src, size_t size)
  * return: NULL when malloc failed, a pointer that the caller must free
  * otherwise.
  */
-static char *get_hmac_file(const char *filename)
+static char *get_hmac_file(const char *filename, const char *subdir)
 {
-	size_t i, filelen, basenamestart = 0;
+	size_t i, filelen, pathlen, namelen, basenamestart = 0;
 	size_t prefixlen = strlen(CHECK_PREFIX);
 	size_t suffixlen = strlen(CHECK_SUFFIX);
 	char *cursor, *checkfile = NULL;
@@ -360,20 +365,35 @@ static char *get_hmac_file(const char *filename)
 		fprintf(stderr, "File too long\n");
 		return NULL;
 	}
-	checkfile = malloc(filelen + prefixlen + 1 + suffixlen);
-	if (!checkfile)
-		return NULL;
-
 	for (i = 0; i < filelen; i++) {
 		if (!strncmp(filename + i, "/", 1))
 			basenamestart = i + 1;
 	}
 
+	namelen = filelen - basenamestart;
+#ifdef CHECK_DIR
+	pathlen = strlen(CHECK_DIR"/") + strlen(subdir) + 1;
+#else
+	(void)subdir; // avoid parameter unused warning
+	pathlen = basenamestart;
+#endif
+
+	checkfile = malloc(pathlen + namelen + prefixlen + 1 /* "." */ +
+		suffixlen + 1 /* null character */);
+	if (!checkfile)
+		return NULL;
+
 	cursor = checkfile;
-	if (basenamestart > 0)
-		cursor = paste(cursor, filename, basenamestart);
+#ifdef CHECK_DIR
+	cursor = paste(cursor, CHECK_DIR"/", strlen(CHECK_DIR"/"));
+	cursor = paste(cursor, subdir, strlen(subdir));
+	cursor = paste(cursor, "/", 1);
+#else
+	if (pathlen > 0)
+		cursor = paste(cursor, filename, pathlen);
+#endif
 	cursor = paste(cursor, CHECK_PREFIX, prefixlen);
-	cursor = paste(cursor, filename + basenamestart, filelen - basenamestart);
+	cursor = paste(cursor, filename + basenamestart, namelen);
 	cursor = paste(cursor, "."CHECK_SUFFIX, 1 + suffixlen);
 	strncpy(cursor, "\0", 1);
 	return checkfile;
@@ -410,7 +430,8 @@ static int hash_files(const struct hash_params *params,
 			const char *filename = filenames[i];
 
 			if (fipshmac) {
-				char *outfile = get_hmac_file(filenames[i]);
+				char *outfile = get_hmac_file(filenames[i],
+				                              params->key.subdir);
 
 				if (!outfile) {
 					fprintf(stderr,
@@ -558,6 +579,14 @@ static int process_checkfile(const struct hash_params *params,
 			goto out;
 		}
 
+		/* fipscheck does not have the filename in the check file */
+		if (targetfile) {
+			ret = hasher(handle, params, targetfile,
+			             hexhash, hexhashlen, stdout);
+			checked_any = 1;
+			goto out;
+		}
+
 		if (filename) {
 			int r;
 
@@ -583,19 +612,8 @@ static int process_checkfile(const struct hash_params *params,
 				if (ret >= 0)
 					ret++;
 			}
-		} else {
-			/*
-			 * fipscheck does not have the filename in the check
-			 * file
-			 */
-			if (targetfile) {
-				ret = hasher(handle, params, targetfile,
-				             hexhash, hexhashlen + 1, stdout);
-				goto out;
-			}
+			checked_any = 1;
 		}
-
-		checked_any = 1;
 	}
 
 out:
@@ -611,7 +629,13 @@ out:
 
 }
 
-static int fipscheck_self(const struct hash_params *params, int just_print)
+/* self-check modes: */
+#define SELFCHECK_CHECK		0
+#define SELFCHECK_PRINT_SELF	1
+#define SELFCHECK_PRINT_LIB	2
+
+static int fipscheck_self(const struct hash_params *params_bin,
+                          const struct hash_params *params_lib, int mode)
 {
 	char *checkfile = NULL;
 	uint32_t n = 0;
@@ -625,9 +649,9 @@ static int fipscheck_self(const struct hash_params *params, int just_print)
 	void *dl = NULL, *sym;
 
 #ifdef HAVE_SECURE_GETENV
-	if (secure_getenv("KCAPI_HASHER_FORCE_FIPS") || just_print) {
+	if (secure_getenv("KCAPI_HASHER_FORCE_FIPS") || mode != SELFCHECK_CHECK) {
 #else
-	if (getenv("KCAPI_HASHER_FORCE_FIPS") || just_print) {
+	if (getenv("KCAPI_HASHER_FORCE_FIPS") || mode != SELFCHECK_CHECK) {
 #endif
 		fipsflag[0] = 1;
 	} else {
@@ -659,59 +683,69 @@ static int fipscheck_self(const struct hash_params *params, int just_print)
 	}
 
 	/* Integrity check of our application. */
-	memset(selfname, 0, sizeof(selfname));
-	selfnamesize = readlink("/proc/self/exe", selfname, BUFSIZE);
-	if (selfnamesize >= BUFSIZE || selfnamesize < 0) {
-		fprintf(stderr, "Cannot obtain my filename\n");
-		ret = -EFAULT;
-		goto out;
-	}
+	if (mode == SELFCHECK_CHECK || mode == SELFCHECK_PRINT_SELF) {
+		memset(selfname, 0, sizeof(selfname));
+		selfnamesize = readlink("/proc/self/exe", selfname, BUFSIZE);
+		if (selfnamesize >= BUFSIZE || selfnamesize < 0) {
+			fprintf(stderr, "Cannot obtain my filename\n");
+			ret = -EFAULT;
+			goto out;
+		}
 
-	if (just_print) {
-		ret = hash_files(params, names, 1, 0, 1);
-		goto out;
-	}
+		if (mode == SELFCHECK_PRINT_SELF) {
+			ret = hash_files(params_bin, names, 1, 0, 1);
+			goto out;
+		}
 
-	checkfile = get_hmac_file(selfname);
-	if (!checkfile) {
-		ret = -ENOMEM;
-		goto out;
-	}
+		checkfile = get_hmac_file(selfname, params_bin->key.subdir);
+		if (!checkfile) {
+			ret = -ENOMEM;
+			goto out;
+		}
 
-	ret = process_checkfile(params, checkfile, selfname, CHK_STATUS);
-	if (ret)
-		goto out;
+		ret = process_checkfile(params_bin, checkfile, selfname, CHK_STATUS);
+		if (ret)
+			goto out;
+	}
 
 	/* Integrity check of shared libkcapi.so file. */
-	memset(selfname, 0, sizeof(selfname));
-	snprintf(selfname, (sizeof(selfname) - 1), "libkcapi.so.%u",
-		 KCAPI_MAJVERSION);
-	dl = dlopen(selfname, RTLD_NODELETE|RTLD_NOLOAD|RTLD_LAZY);
-	if (dl == NULL) {
-		fprintf(stderr, "dlopen of file %s failed\n", selfname);
-		ret = -EFAULT;
-		goto out;
+	if (mode == SELFCHECK_CHECK || mode == SELFCHECK_PRINT_LIB) {
+		memset(selfname, 0, sizeof(selfname));
+		snprintf(selfname, (sizeof(selfname) - 1), "libkcapi.so.%u",
+		         KCAPI_MAJVERSION);
+		dl = dlopen(selfname, RTLD_NODELETE|RTLD_NOLOAD|RTLD_LAZY);
+		if (dl == NULL) {
+			fprintf(stderr, "dlopen of file %s failed\n", selfname);
+			ret = -EFAULT;
+			goto out;
+		}
+
+		memset(selfname, 0, sizeof(selfname));
+		sym = dlsym(dl, "kcapi_md_init");
+		if (sym == NULL || !dladdr(sym, &info)) {
+			fprintf(stderr, "finding symbol kcapi_md_init failed\n");
+			ret = -EFAULT;
+			goto out;
+		}
+
+		strncpy(selfname, info.dli_fname, (sizeof(selfname) - 1));
+
+		if (mode == SELFCHECK_PRINT_LIB) {
+			ret = hash_files(params_lib, names, 1, 0, 1);
+			goto out;
+		}
+
+		if (checkfile)
+			free(checkfile);
+		checkfile = get_hmac_file(selfname, params_lib->key.subdir);
+		if (!checkfile) {
+			ret = -ENOMEM;
+			goto out;
+		}
+
+		ret = process_checkfile(params_lib, checkfile, selfname, CHK_STATUS);
 	}
 
-	memset(selfname, 0, sizeof(selfname));
-	sym = dlsym(dl, "kcapi_md_init");
-	if (sym == NULL || !dladdr(sym, &info)) {
-		fprintf(stderr, "finding symbol kcapi_md_init failed\n");
-		ret = -EFAULT;
-		goto out;
-	}
-
-	strncpy(selfname, info.dli_fname, (sizeof(selfname) - 1));
-
-	free(checkfile);
-	checkfile = get_hmac_file(selfname);
-	if (!checkfile) {
-		ret = -ENOMEM;
-		goto out;
-	}
-
-	ret = process_checkfile(params, checkfile, selfname, CHK_STATUS);
-
 out:
 	if (checkfile)
 		free(checkfile);
@@ -735,7 +769,7 @@ int main(int argc, char *argv[])
 	const struct hash_name *names;
 	struct hash_params params = {
 		.name = { NULL, NULL },
-		.key = { NULL, 0 },
+		.key = { NULL, NULL, 0 },
 		.hashlen = 0,
 		.bsd_style = 0,
 	};
@@ -748,13 +782,20 @@ int main(int argc, char *argv[])
 	const char *targetfile = NULL;
 	uint8_t *hmackey_alloc = NULL;
 	uint8_t *hmackey_mmap = NULL;
+	int opt_index = 0;
 	int loglevel = 0;
 	int hmac = 0;
 	int fipscheck = 0;
 	int fipshmac = 0;
-	int print_self_hash = 0;
+	int selfcheck_mode = SELFCHECK_CHECK;
 
-	static const char *opts_short = "c:uh:t:Sqk:K:vbd:P";
+	static const char *opts_name_short = "n:";
+	static const struct option opts_name[] = {
+		{"name", 1, 0, 'n'},
+		{0, 0, 0, 0}
+	};
+
+	static const char *opts_short = "c:uh:t:SLqk:K:vbd:P";
 	static const struct option opts[] = {
 		{"help", 0, 0, 0},
 		{"tag", 0, 0, 0},
@@ -764,6 +805,7 @@ int main(int argc, char *argv[])
 		{"hash", 1, 0, 'h'},
 		{"truncate", 1, 0, 't'},
 		{"self-sum", 0, 0, 'S'},
+		{"self-sum-lib", 0, 0, 'L'},
 		{"status", 0, 0, 'q'},
 		{"key-file", 1, 0, 'k'},
 		{"key", 1, 0, 'K'},
@@ -799,6 +841,11 @@ int main(int argc, char *argv[])
 	}
 	basen = basename(basec);
 
+	if (getopt_long(argc, argv, opts_name_short, opts_name, &opt_index) == 'n')
+		basen = optarg;
+	else
+		opt_index = 0;
+
 	params_self = &PARAMS_SELF_FIPSCHECK;
 	if (0 == strncmp(basen, "sha256sum", 9)) {
 		names = NAMES_SHA256;
@@ -854,7 +901,6 @@ int main(int argc, char *argv[])
 	}
 
 	while (1) {
-		int opt_index = 0;
 		int c = getopt_long(argc, argv, opts_short, opts, &opt_index);
 		
 		if (-1 == c)
@@ -931,7 +977,10 @@ int main(int argc, char *argv[])
 				params.hashlen /= 8;
 				break;
 			case 'S':
-				print_self_hash = 1;
+				selfcheck_mode = SELFCHECK_PRINT_SELF;
+				break;
+			case 'L':
+				selfcheck_mode = SELFCHECK_PRINT_LIB;
 				break;
 			case 'q':
 				loglevel = CHK_STATUS;
@@ -997,31 +1046,34 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (print_self_hash) {
+	if (selfcheck_mode != SELFCHECK_CHECK) {
 		if (checkfile) {
-			fprintf(stderr, "-S and -c cannot be combined\n");
+			fprintf(stderr, "-S/-L and -c cannot be combined\n");
 			ret = 1;
 			goto out;
 		}
 		if (optind != argc) {
-			fprintf(stderr, "-S cannot be used with input files\n");
+			fprintf(stderr, "-S/-L cannot be used with input files\n");
 			ret = 1;
 			goto out;
 		}
 	}
 
-	if (fipscheck_self(params_self, print_self_hash)) {
+	/* library self-check must be consistent across apps: */
+	if (fipscheck_self(params_self, &PARAMS_SELF_FIPSCHECK, selfcheck_mode)) {
 		fprintf(stderr, "Integrity check of application %s failed\n",
 			basen);
 		ret = 1;
 		goto out;
 	}
 
-	if (print_self_hash) {
+	if (selfcheck_mode != SELFCHECK_CHECK) {
 		ret = 0;
 		goto out;
 	}
 
+	params.name = names[hmac];
+
 	if (fipscheck) {
 		if (optind >= argc) {
 			fprintf(stderr, "No file to check given for fipscheck\n");
@@ -1037,7 +1089,7 @@ int main(int argc, char *argv[])
 		targetfile = argv[optind];
 		if (checkfile)
 			free(checkfile);
-		checkfile = get_hmac_file(targetfile);
+		checkfile = get_hmac_file(targetfile, params.key.subdir);
 		if (!checkfile) {
 			ret = 1;
 			goto out;
@@ -1045,8 +1097,6 @@ int main(int argc, char *argv[])
 		optind++;
 	}
 
-	params.name = names[hmac];
-
 	if (!checkfile)
 		ret = hash_files(&params, argv + optind, (argc - optind),
 		                 fipshmac, 0);
