@@ -1,6 +1,6 @@
 #!/bin/bash
 #
-# Copyright (C) 2017, Stephan Mueller <smueller@chronox.de>
+# Copyright (C) 2017 - 2018, Stephan Mueller <smueller@chronox.de>
 #
 # License: see LICENSE file in root directory
 #
@@ -43,7 +43,7 @@ fi
 
 # TESTKERNELS specifies the kernel versions (i.e. Linux kernel
 # source code directories) to be used and tested
-TESTKERNELS="linux-4.13 linux-4.12 linux-4.10 linux-4.7 linux-4.5 linux-4.4.86 linux-4.3.6"
+TESTKERNELS="linux-4.17 linux-4.13 linux-4.12 linux-4.10 linux-4.7 linux-4.5 linux-4.4.86 linux-4.3.6"
 
 ###################################################################
 # General variables - do not change
